%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                  Optimal Growth model			              %
%                                                             %
%             Collocation method (AR(1) case)                 %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
global nbk kmin ksup XK kt;
global nba amin asup XA at;
global nstate nodea nodek wmat wgrid;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Algorithm Parameters                        %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nbk=4;				% Degree of polynomials (capital)
nba=2;				% Degree of polynomials (technology shock)
ncoef=(nbk+1)*(nba+1);	% # of coefficients
nodek=nbk+1;		% # of Nodes
nodea=nba+1;		% # of Nodes
nstate=12;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                    Structural Parameters                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
delta	= 0.1;
beta	= 0.95;
alpha	= 0.3;
sigma	= 1.5;
ysk	=(1-beta*(1-delta))/(alpha*beta);
ksy	= 1/ysk;
ys		= ksy^(alpha/(1-alpha));
ks		= ys^(1/alpha);
is		= delta*ks;
cs		= ys-is;
ab		= 0;
%
% Technology shock process
%
rho	= 0.8;
se		= 0.2;
ma		= 0;
[wgrid,wmat]=hernodes(nstate);
wgrid=wgrid*sqrt(2)*se; 
%
% grid for the income
%
amin=(ma+wgrid(nstate));%/(1-rho));
asup=(ma+wgrid(1));%/(1-rho));
ra=rcheb(nodea);						    % roots
at=itransfo(ra,amin,asup);					% grid
XA=cheb(ra,[0:nba]);						% Polynomials
%
% grid for the capital stock
%
kmin	= log(.1);
ksup	= log(6);
rk		= rcheb(nodek);						% roots
kt		= exp(itransfo(rk,kmin,ksup));	% grid
XK		= cheb(rk,[0:nbk]);					% Polynomials
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                     Initial Conditions                      %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
a0=[ 
  -0.23759592487257
   0.60814488103911
   0.03677400318790
   0.69025680170443
  -0.21654209984197
   0.00551243342828
   0.03499834613714
  -0.00341171507904
  -0.00449139656933
   0.00085302605779
   0.00285737302122
  -0.00002348542016
  -0.00011606672164
  -0.00003323351559
   0.00018045618825];
a0=a0(:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                          Main loop                          %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param=[alpha beta delta sigma ma rho]';
th=fcsolve('residuals',a0,[],param);
disp('Final rule : ');
disp(th')

lt		= length(th);
nk	= 20;
na	= 10;
kt		= [kmin:(ksup-kmin)/(nk-1):ksup]';
at		= [amin:(asup-amin)/(na-1):asup]';
rk		= transfo(kt,kmin,ksup);
ra		= transfo(at,kmin,asup);
XT		= cheb(rk(:),[0:nbk]);
XA		= cheb(ra(:),[0:nba]);
kt		= exp(kt);
ct=zeros(nk,na);
k1=zeros(nk,na);
ii=[];
for i=1:nk;
	for j=1:na;
	XX0	= makepoly(XA(j,:),XT(i,:));
   ct(i,j)		= exp(XX0*th);
   k1(i,j)		= exp(at(j))*kt(i).^alpha+(1-delta)*kt(i)-ct(i,j);
end
end


subplot(221);h=mesh(at,kt,k1);
view(-50,12);
xlabel('a_t','fontname','times','fontsize',12)
ylabel('k_t','fontname','times','fontsize',12)
title('Next period capital stock','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(222);h=mesh(at,kt,ct);
view(-50,12);
xlabel('a_t','fontname','times','fontsize',12)
ylabel('k_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause;
print -depsc2 drcollocg.eps
close;