%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                  Optimal Growth model			              %
%                                                             %
%             Galerkin method (AR(1) case)                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
global nbk kmin ksup XK kt;
global nba amin asup XA at;
global nstate nodea nodek wmat wgrid;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Algorithm Parameters                        %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nbk=4;				% Degree of polynomials (capital)
nba=2;				% Degree of polynomials (technology shock)
ncoef=nbk+1;		% # of coefficients
nodek=20;			% # of Nodes
nodea=10;			% # of Nodes
nstate=12;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                    Structural Parameters                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
delta	= 0.1;
beta	= 0.95;
alpha	= 0.3;
sigma	= 1.5;
ysk	=(1-beta*(1-delta))/(alpha*beta);
ksy	= 1/ysk;
ys		= ksy^(alpha/(1-alpha));
ks		= ys^(1/alpha);
is		= delta*ks;
cs		= ys-is;
ab		= 0;
%
% Technology shock process
%
rho	= 0.8;
se		= 0.2;
ma		= 0;
[wgrid,wmat]=hernodes(nstate);
wgrid=wgrid*sqrt(2)*se; 
%
% grid for the income
%
amin=(ma-wgrid(1));%/(1-rho));
asup=(ma+wgrid(1));%/(1-rho));
ra=rcheb(nodea);								% roots
at=itransfo(ra,amin,asup);					% grid
XA=cheb(ra,[0:nba]);							% Polynomials
%
% grid for the capital stock
%
kmin	= log(0.1);
ksup	= log(6);
rk		= rcheb(nodek);						% roots
kt		= exp(itransfo(rk,kmin,ksup));	% grid
XK		= cheb(rk,[0:nbk]);					% Polynomials
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                     Initial Conditions                      %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
a0=[ 
  -0.23759592487257
   0.60814488103911
   0.03677400318790
   0.69025680170443
  -0.21654209984197
   0.00551243342828
   0.03499834613714
  -0.00341171507904
  -0.00449139656933
   0.00285737302122
  -0.00002348542016
  -0.00011606672164
];

a0=a0(:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                          Main loop                          %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param=[alpha beta delta sigma ma rho]';
th=fcsolve('residuals',a0,[],param);
disp('Final rule : ');
disp(th')

lt		= length(th);
nb		= 100;
kt		= [kmin:(ksup-kmin)/(nb-1):ksup]';
rk		= transfo(kt,kmin,ksup);
XT		= cheb(rk(:),[0:nbk]);
kt		= exp(kt);
c=[];
k1=[];
ii=[];
for j=1:nodea;
   XX0	= makepoly(XA(j,:),XT);
   c		= [c exp(XX0*th)];
   k1		= [k1 exp(at(j))*kt.^alpha+(1-delta)*kt-c(:,j)];
   ii       = [ii exp(at(j))*kt.^alpha-c(:,j)];
end
plot(kt,k1);
pause;close
plot(kt,ii);
pause;close