%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                  Optimal Growth model			              %
%                                                             %
%            Galerkin method (Markov Chain case)              %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
global kmin ksup XX kt;
global nstate nbk ncoef XX XT PI;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Algorithm Parameters                        %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nbk=4;				% Degree of polynomials
nodes=20;			% # of Nodes
nstate=2;			% # of possible states for the income process
ncoef=nbk+1;		% # of coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                    Structural Parameters                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
delta	= 0.1;
beta	= 0.95;
alpha	= 0.3;
sigma	= 1.5;
ysk	=(1-beta*(1-delta))/(alpha*beta);
ksy	= 1/ysk;
ys		= ksy^(alpha/(1-alpha));
ks		= ys^(1/alpha);
is		= delta*ks;
cs		= ys-is;
%
% Markov Chain: technology shock
%
rho	= 0.8;
se		= 0.2;
ma		= 0;
[agrid,wmat]=hernodes(nstate);
agrid=agrid*sqrt(2)*se; 
PI=transprob(agrid,wmat,0,rho,se); 
at=agrid+ma;
%
% grid for the capital stock
%
kmin	= log(0.1);
ksup	= log(6);
rk		= rcheb(nodes);					% roots
kt		= exp(itransfo(rk,kmin,ksup));		% grid
XX		= cheb(rk,[0:nbk]);				% Polynomials
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                     Initial Conditions                      %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
a0=repmat([-0.2 0.65 0.04 0 0]',nstate,1);
a0=a0(:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                          Main loop                          %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param=[alpha beta delta sigma]';
th=fcsolve('residuals',a0,[],param,at);
disp('Final rule : ');
th		= reshape(th,ncoef,nstate);
disp(th')

lt		= length(th);
nb		= 1000;
kt		= [kmin:(ksup-kmin)/(nb-1):ksup]';
rk		= transfo(kt,kmin,ksup);
XX		= cheb(rk(:),[0:nbk]);
kt      = exp(kt);
ct=[];k1=[];ii=[];
for i=1:nstate;
   ct		= [ct exp(XX*th(:,i))];
   k1		= [k1 exp(at(i))*kt.^alpha+(1-delta)*kt-ct(:,i)];
   ii		= [ii exp(at(i))*kt.^alpha-ct(:,i)];
end;

subplot(221);h=plot(kt,k1);
set(h,'color','k','linewidth',1);
xlim=get(gca,'xlim');
ylim=get(gca,'ylim');
h=line(xlim,xlim);
set(h,'color','k','linestyle','--');
xlabel('k_t','fontname','times','fontsize',12)
title('Next period capital stock','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(222);plot(kt,ct,'color','k','linewidth',1);
xlabel('k_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause
print -depsc2 drgal.eps
close
