%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                  Optimal Growth model			              %
%                                                             %
%             PEA Galerkin method (AR(1) case)                %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
global nbk kmin ksup XK kt;
global nba amin asup XA at;
global nstate nodea nodek wmat wgrid;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Algorithm Parameters                        %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nbk=4;				% Degree of polynomials (capital)
nba=2;				% Degree of polynomials (technology shock)
ncoef=nbk+1;		% # of coefficients
nodek=20;			% # of Nodes
nodea=10;			% # of Nodes
nstate=12;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                    Structural Parameters                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
delta	= 0.1;
beta	= 0.95;
alpha	= 0.3;
sigma	= 1.5;
ysk	=(1-beta*(1-delta))/(alpha*beta);
ksy	= 1/ysk;
ys		= ksy^(alpha/(1-alpha));
ks		= ys^(1/alpha);
is		= delta*ks;
cs		= ys-is;
ab		= 0;
%
% Technology shock process
%
rho	= 0.8;
se		= 0.05;
ma		= 0;
[wgrid,wmat]=hernodes(nstate);
wgrid=wgrid*sqrt(2)*se; 
%
% grid for the income
%
amin=(ma-wgrid(1));%/(1-rho));
asup=(ma+wgrid(1));%/(1-rho));
ra=rcheb(nodea);								% roots
at=itransfo(ra,amin,asup);					% grid
XA=cheb(ra,[0:nba]);							% Polynomials
%
% grid for the capital stock
%
kmin	= log(1);
ksup	= log(6);
rk		= rcheb(nodek);						% roots
kt		= exp(itransfo(rk,kmin,ksup));	% grid
XK		= cheb(rk,[0:nbk]);					% Polynomials
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                     Initial Conditions                      %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
a0=[  -0.078850
      -0.190360
      -0.003811
      -0.598213
       0.046219
       0.000182
      -0.016561
      -0.000595
       0.000111
       0.000004
      -0.000163
       0.000014
    ];
a0=a0(:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                          Main loop                          %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param=[alpha beta delta sigma ma rho]';
th=fcsolve('residuals',a0,[],param);
disp('Final rule : ');
disp(th')

lt		= length(th);
nb		= 100;
kt		= [kmin:(ksup-kmin)/(nb-1):ksup]';
rk		= transfo(kt,kmin,ksup);
XT		= cheb(rk(:),[0:nbk]);
kt		= exp(kt);
c=[];
k1=[];
for j=1:nodea;
   XX0	= makepoly(XA(j,:),XT);
   Ephi	= exp(XX0*th);
   c		= [c Ephi.^(-1/sigma)];
   k1		= [k1 exp(at(j))*kt.^alpha+(1-delta)*kt-Ephi.^(-1/sigma)];
end
plot(kt,k1);
pause;close
plot(kt,c);
pause;close