%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                  Optimal Growth model			              %
%                                                             %
%           PEA Galerkin method (Markov Chain case)           %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
global kmin ksup XX kt;
global nstate nbk ncoef XX XT PI;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Algorithm Parameters                        %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nbk=4;				% Degree of polynomials
nodes=20;			% # of Nodes
nstate=10;			% # of possible states for the income process
ncoef=nbk+1;		% # of coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                    Structural Parameters                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
delta	= 0.1;
beta	= 0.95;
alpha	= 0.3;
sigma	= 1.5;
ysk	=(1-beta*(1-delta))/(alpha*beta);
ksy	= 1/ysk;
ys		= ksy^(alpha/(1-alpha));
ks		= ys^(1/alpha);
is		= delta*ks;
cs		= ys-is;
%
% Markov Chain: technology shock
%
rho	= 0.8;
se		= 0.05;
ma		= 0;
[agrid,wmat]=hernodes(nstate);
agrid=agrid*sqrt(2)*se; 
PI=transprob(agrid,wmat,0,rho,se); 
at=agrid+ma;
%
% grid for the capital stock
%
kmin	= log(1);
ksup	= log(6);
rk		= rcheb(nodes);					% roots
kt		= itransfo(rk,kmin,ksup);		% grid
XX		= cheb(rk,[0:nbk]);				% Polynomials
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                     Initial Conditions                      %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
a0=repmat([0.05 -0.4 -0.01 0 0]',nstate,1);
a0=a0(:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                          Main loop                          %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param=[alpha beta delta sigma]';
th=fcsolve('residuals',a0,[],param,at);
disp('Final rule : ');
th		= reshape(th,ncoef,nstate);
disp(th')

lt		= length(th);
nb		= 1000;
kt		= [kmin:(ksup-kmin)/(nb-1):ksup]';
rk		= transfo(kt,kmin,ksup);
XX		= cheb(rk(:),[0:nbk]);
c=[];k1=[];
for i=1:nstate;
   Ephi	= exp(XX*th(:,i));
   c		= [c Ephi.^(-1/sigma)];
   k1		= [k1 exp(at(i))*exp(kt).^alpha+(1-delta)*exp(kt)-Ephi.^(-1/sigma)];
end;
plot(exp(kt),[k1 exp(kt)]);
pause;close