%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%     Optimal Growth model with irreversible investment		  %
%                                                             %
%              PEA Galerkin method (AR(1) case)               %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
global nbk kmin ksup XK kt;
global nba amin asup XA at;
global nstate nodea nodek wmat wgrid;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Algorithm Parameters                        %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nbk=8;				% Degree of polynomials (capital)
nba=2;				% Degree of polynomials (technology shock)
ncoef=nbk+1;		% # of coefficients
nodek=20;			% # of Nodes
nodea=10;			% # of Nodes
ncoef=24;
nstate=12;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                    Structural Parameters                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
delta	= 0.1;
beta	= 0.95;
alpha	= 0.3;
sigma	= 1.5;
ysk	    =(1-beta*(1-delta))/(alpha*beta);
ksy	    = 1/ysk;
ys		= ksy^(alpha/(1-alpha));
ks		= ys^(1/alpha);
is		= delta*ks;
cs		= ys-is;
%
% Technology shock process
%
rho 	= 0.8;
se	    = 0.05;
ma		= 0;
[wgrid,wmat]=hernodes(nstate);
wgrid=wgrid*sqrt(2)*se; 
%
% grid for the shock
%
amin=(ma-wgrid(1));%/(1-rho));
asup=(ma+wgrid(1));%/(1-rho));
ra=rcheb(nodea);								% roots
at=itransfo(ra,amin,asup);					% grid
XA=cheb(ra,[0:nba]);							% Polynomials
%
% grid for the capital stock
%
kmin	= log(1);
ksup	= log(6);
rk		= rcheb(nodek);						% roots
kt		= exp(itransfo(rk,kmin,ksup));	% grid
XK		= cheb(rk,[0:nbk]);					% Polynomials
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                     Initial Conditions                      %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
a0=[
  -0.08505040914339
  -0.17315912599760
  -0.01354994729012
  -0.60992884172776
   0.07827018769200
  -0.01805188106916
  -0.02659382901031
   0.02533330630680
  -0.01467181898688
  -0.00757633564818
   0.01826044877906
  -0.01019976212860
  -0.00478099731412
   0.01120296359356
  -0.00602907736458
  -0.00247873828736
   0.00553028670647
  -0.00288720511726
  -0.00125206161326
   0.00220171661431
  -0.00095301121856
  -0.00059978820237
   0.00071109130262
  -0.00019950708098
];
a0=a0(:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                          Main loop                          %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param=[alpha beta delta sigma ma rho]';
th=fcsolve('residuals',a0,[],param);
disp('Final rule : ');
disp(th')

lt		= length(th);
nb		= 100;
kt		= [kmin:(ksup-kmin)/(nb-1):ksup]';
rk		= transfo(kt,kmin,ksup);
XT		= cheb(rk(:),[0:nbk]);
kt		= exp(kt);
c=[];
k1=[];
ii=[];
for j=1:nodea;
    XX0	= makepoly(XA(j,:),XT);
    Ephi= exp(XX0*th);
    y	= exp(at(j))*kt.^alpha;
    c		= [c Ephi.^(-1/sigma)];
    iv	= y-Ephi.^(-1/sigma);
    dum	= iv>0;
    ii=[ii iv.*dum];
    k1  = [k1 (1-delta)*kt+iv.*dum];
end


subplot(221);plot(kt,k1);
xlabel('k_t','fontname','times','fontsize',12)
ylabel('k_{t+1}','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(222);plot(kt,ii);
xlabel('k_t','fontname','times','fontsize',12)
ylabel('i_t','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause;
print -depsc2 irpeamg.eps
close